@@ -14,17 +14,17 @@
 #    under the License.
 
 from __future__ import print_function
-import six
-import sys
-import textwrap
-import uuid
 
+import os
+import textwrap
+
+from oslo_serialization import jsonutils
+from oslo_utils import encodeutils
+from oslo_utils import importutils
 import prettytable
+import six
 
 from ceilometerclient import exc
-from ceilometerclient.openstack.common import cliutils
-from ceilometerclient.openstack.common import importutils
-from ceilometerclient.openstack.common import strutils
 
 
 # Decorator for cli-args
@@ -44,11 +44,22 @@ def arg(*args, **kwargs):
     return _decorator
 
 
-def pretty_choice_list(l):
-    return ', '.join("'%s'" % i for i in l)
+def print_list(objs, fields, field_labels, formatters=None, sortby=0):
+    """Print a list of objects as a table, one row per object.
 
+    :param objs: Iterable of :class:`Resource`
+    :param fields: Attributes that correspond to columns, in order
+    :param field_labels: Labels to use in the heading of the table, default to
+                         fields.
+    :param formatters: `dict` of callables for field formatting
+    :param sortby: Index of the field for sorting table rows
+    """
+    formatters = formatters or {}
 
-def print_list(objs, fields, field_labels, formatters={}, sortby=0):
+    if len(field_labels) != len(fields):
+        raise ValueError(("Field labels list %(labels)s has different number "
+                          "of elements than fields list %(fields)s"),
+                         {'labels': field_labels, 'fields': fields})
 
     def _make_default_formatter(field):
         return lambda o: getattr(o, field, '')
@@ -60,9 +71,25 @@ def print_list(objs, fields, field_labels, formatters={}, sortby=0):
         else:
             new_formatters[field_label] = _make_default_formatter(field)
 
-    cliutils.print_list(objs, field_labels,
-                        formatters=new_formatters,
-                        sortby_index=sortby)
+    kwargs = {} if sortby is None else {'sortby': field_labels[sortby]}
+    pt = prettytable.PrettyTable(field_labels)
+    pt.align = 'l'
+
+    for o in objs:
+        row = []
+        for field in field_labels:
+            if field in new_formatters:
+                row.append(new_formatters[field](o))
+            else:
+                field_name = field.lower().replace(' ', '_')
+                data = getattr(o, field_name, '')
+                row.append(data)
+        pt.add_row(row)
+
+    if six.PY3:
+        print(encodeutils.safe_encode(pt.get_string(**kwargs)).decode())
+    else:
+        print(encodeutils.safe_encode(pt.get_string(**kwargs)))
 
 
 def nested_list_of_dict_formatter(field, column_names):
@@ -83,15 +110,12 @@ def format_nested_list_of_dict(l, column_names):
 
 
 def print_dict(d, dict_property="Property", wrap=0):
-    pt = prettytable.PrettyTable([dict_property, 'Value'],
-                                 caching=False, print_empty=False)
+    pt = prettytable.PrettyTable([dict_property, 'Value'], print_empty=False)
     pt.align = 'l'
     for k, v in sorted(six.iteritems(d)):
         # convert dict to str to check length
-        if isinstance(v, dict):
-            v = str(v)
-        if isinstance(v, six.string_types):
-            v = strutils.safe_encode(v)
+        if isinstance(v, (list, dict)):
+            v = jsonutils.dumps(v)
         # if value has a newline, add in multiple rows
         # e.g. fault with stacktrace
         if v and isinstance(v, six.string_types) and r'\n' in v:
@@ -99,39 +123,18 @@ def print_dict(d, dict_property="Property", wrap=0):
             col1 = k
             for line in lines:
                 if wrap > 0:
-                    line = textwrap.fill(str(line), wrap)
+                    line = textwrap.fill(six.text_type(line), wrap)
                 pt.add_row([col1, line])
                 col1 = ''
         else:
             if wrap > 0:
-                v = textwrap.fill(str(v), wrap)
+                v = textwrap.fill(six.text_type(v), wrap)
             pt.add_row([k, v])
-    print(pt.get_string())
-
-
-def find_resource(manager, name_or_id):
-    """Helper for the _find_* methods."""
-    # first try to get entity as integer id
-    try:
-        if isinstance(name_or_id, int) or name_or_id.isdigit():
-            return manager.get(int(name_or_id))
-    except exc.NotFound:
-        pass
-
-    # now try to get entity as uuid
-    try:
-        uuid.UUID(str(name_or_id))
-        return manager.get(name_or_id)
-    except (ValueError, exc.NotFound):
-        pass
-
-    # finally try to find entity by name
-    try:
-        return manager.find(name=name_or_id)
-    except exc.NotFound:
-        msg = "No %s with a name or ID of '%s' exists." % \
-              (manager.resource_class.__name__.lower(), name_or_id)
-        raise exc.CommandError(msg)
+    encoded = encodeutils.safe_encode(pt.get_string())
+    # FIXME(gordc): https://bugs.launchpad.net/oslo-incubator/+bug/1370710
+    if six.PY3:
+        encoded = encoded.decode()
+    print(encoded)
 
 
 def import_versioned_module(version, submodule=None):
@@ -155,8 +158,7 @@ def args_array_to_dict(kwargs, key_to_convert):
 
 
 def args_array_to_list_of_dicts(kwargs, key_to_convert):
-    """Converts ['a=1;b=2','c=3;d=4'] to [{a:1,b:2},{c:3,d:4}]
-    """
+    """Converts ['a=1;b=2','c=3;d=4'] to [{a:1,b:2},{c:3,d:4}]."""
     values_to_convert = kwargs.get(key_to_convert)
     if values_to_convert:
         try:
@@ -195,7 +197,13 @@ def merge_nested_dict(dest, source, depth=0):
             dest[key] = value
 
 
-def exit(msg=''):
-    if msg:
-        print(msg, file=sys.stderr)
-    sys.exit(1)
+def env(*args, **kwargs):
+    """Returns the first environment variable set.
+
+    If all are empty, defaults to '' or keyword arg `default`.
+    """
+    for arg in args:
+        value = os.environ.get(arg)
+        if value:
+            return value
+    return kwargs.get('default', '')
