@@ -1,4 +1,3 @@
-# -*- encoding: utf-8 -*-
 # Copyright 2014 Hewlett-Packard Development Company, L.P.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -21,13 +20,19 @@ class Event(base.Resource):
     def __repr__(self):
         return "<Event %s>" % self._info
 
+    def __getattr__(self, k):
+        if k == 'id':
+            return self.message_id
+        return super(Event, self).__getattr__(k)
+
 
 class EventManager(base.Manager):
     resource_class = Event
 
-    def list(self, q=None):
+    def list(self, q=None, limit=None):
         path = '/v2/events'
-        return self._list(options.build_url(path, q))
+        params = ['limit=%s' % limit] if limit else None
+        return self._list(options.build_url(path, q, params))
 
     def get(self, message_id):
         path = '/v2/events/%s'
